#!/bin/awk -f

# frequency.awk: calculates the frequency of each word that appears in
# a text file and prints it out (in unsorted order). Leverages awk's
# built-in associative arrays along with several other features.

{                               # match every line
  gsub(/[^a-zA-Z ]/," ");       # eliminate non-word characters like ',' and '!'
  for(i=1; i<=NF; i++){         # iterate over each word
    counts[$i] = counts[$i]+1   # use ith word as a key, increment its frequency
  }                             # leverages built-in semantics: not present -> 0
}
END{                            # after processing all lines
  for(key in counts){           # iterate over all keys in the counts array
    print key,":",counts[key]   # print the key (word) and its count
  }
}
